@@ -5,6 +5,7 @@ from redmine.managers import ResourceManager
 from redmine.utilities import to_string
 from redmine.exceptions import (
     AuthError,
+    ConflictError,
     ImpersonateError,
     ServerError,
     ValidationError,
@@ -26,6 +27,7 @@ class Redmine(object):
         self.impersonate = kwargs.get('impersonate', None)
         self.date_format = kwargs.get('date_format', '%Y-%m-%d')
         self.datetime_format = kwargs.get('datetime_format', '%Y-%m-%dT%H:%M:%SZ')
+        self.raise_attr_exception = kwargs.get('raise_attr_exception', True)
 
     def __getattr__(self, resource):
         """Returns either ResourceSet or Resource object depending on the method used on the ResourceManager"""
@@ -69,16 +71,15 @@ class Redmine(object):
         response = getattr(requests, method)(url, **kwargs)
 
         if response.status_code in (200, 201):
-            if response.status_code == 200:
-                if method == 'put':
-                    return response.text
-                elif method == 'delete':
-                    return True
+            if response.status_code == 200 and method in ('put', 'delete'):
+                return True
             return response.json()
         elif response.status_code == 401:
             raise AuthError()
         elif response.status_code == 404:
             raise ResourceNotFoundError
+        elif response.status_code == 409:
+            raise ConflictError
         elif response.status_code == 412 and self.impersonate is not None:
             raise ImpersonateError()
         elif response.status_code == 422:
