@@ -38,42 +38,52 @@ class ResourceManager(object):
         """A proxy for Redmine object request which does some extra work for resource retrieval"""
         self.params.update(**params)
 
-        # Redmine allows us to only return 100 resources per request, so if
-        # we want to get all or > 100 resources we need to do some extra work
-        if 'limit' in self.params and not 0 < self.params['limit'] <= 100:
-            results = []
-            self.params['offset'] = self.params.get('offset', 0)
+        results = []
+        total_count = 0
+        limit = self.params.get('limit', 0)
+        offset = self.params.get('offset', 0)
 
-            # we want to get all resources
-            if self.params['limit'] == 0:
-                self.params['limit'] = 100
+        if limit == 0:
+            limit = 100
 
-                while True:
-                    response = self.redmine.request('get', self.url, params=self.params)
+        while True:
+            response = self.redmine.request('get', self.url, params=dict(self.params, limit=limit, offset=offset))
 
-                    if response is None:
+            if response is None:
+                break
+
+            # A single resource was requested via get()
+            if isinstance(response[self.container], dict):
+                results = response[self.container]
+                total_count = 1
+                break
+
+            # Resource supports limit/offset on Redmine level
+            if all(param in response for param in ('total_count', 'limit', 'offset')):
+                total_count = response['total_count']
+                results.extend(response[self.container])
+
+                # We want to get all resources
+                if self.params.get('limit', 0) == 0:
+                    offset += limit
+
+                    if total_count <= offset:
                         break
+                # We want to get only some resources
+                else:
+                    limit -= 100
+                    offset += 100
 
-                    results.extend(response[self.container])
-                    self.params['offset'] += self.params['limit']
-
-                    if response.get('total_count', 0) <= self.params['offset']:
+                    if limit < 0:
                         break
-            # we want to get > 100 resources
+            # We have to mimic limit/offset if a resource
+            # doesn't support this feature on Redmine level
             else:
-                while self.params['limit'] > 0:
-                    response = self.redmine.request('get', self.url, params=self.params)
+                total_count = len(response[self.container])
+                results = response[self.container][offset:limit + offset]
+                break
 
-                    if response is None:
-                        break
-
-                    results.extend(response[self.container])
-                    self.params['limit'] -= 100
-                    self.params['offset'] += 100
-
-            return results
-
-        return self.redmine.request('get', self.url, params=self.params)[self.container]
+        return results, total_count
 
     def to_resource(self, resource):
         """Converts a single resource dict from Redmine result set to resource object"""
@@ -83,6 +93,10 @@ class ResourceManager(object):
         """Converts an iterable with resource dicts from Redmine result set to ResourceSet object"""
         return ResourceSet(self, resources)
 
+    def new(self):
+        """Returns new empty resource"""
+        return self.to_resource({})
+
     def get(self, resource_id, **params):
         """Returns a Resource object directly by resource id (can be either integer id or string identifier)"""
         if self.resource_class.query_one is None or self.resource_class.container_one is None:
@@ -93,9 +107,9 @@ class ResourceManager(object):
         except KeyError as exception:
             raise ValidationError('{0} argument is required'.format(exception))
 
-        self.params = params
+        self.params = self.resource_class.translate_params(params)
         self.container = self.resource_class.container_one
-        return self.resource_class(self, self.retrieve())
+        return self.resource_class(self, self.retrieve()[0])
 
     def all(self, **params):
         """Returns a ResourceSet object with all Resource objects"""
@@ -103,7 +117,7 @@ class ResourceManager(object):
             raise ResourceBadMethodError()
 
         self.url = '{0}{1}'.format(self.redmine.url, self.resource_class.query_all)
-        self.params = params
+        self.params = self.resource_class.translate_params(params)
         self.container = self.resource_class.container_all
         return ResourceSet(self)
 
@@ -119,13 +133,9 @@ class ResourceManager(object):
             self.url = '{0}{1}'.format(self.redmine.url, self.resource_class.query_filter.format(**filters))
             self.container = self.resource_class.container_filter.format(**filters)
         except KeyError:
-            if self.resource_class.query_all is not None and self.resource_class.container_all is not None:
-                self.url = '{0}{1}'.format(self.redmine.url, self.resource_class.query_all)
-                self.container = self.resource_class.container_all
-            else:
-                raise ResourceFilterError()
+            raise ResourceFilterError()
 
-        self.params = filters
+        self.params = self.resource_class.translate_params(filters)
         return ResourceSet(self)
 
     def create(self, **fields):
@@ -157,9 +167,9 @@ class ResourceManager(object):
         except ResourceNotFoundError:
             response = self.redmine.request('put', url, data=data)
 
-        if self.container in response:
-            resource = self.resource_class(self, response[self.container])
-        else:
+        try:
+            resource = self.to_resource(response[self.container])
+        except TypeError:
             raise ValidationError('Resource already exists')  # fix for repeated PUT requests
 
         self.params = formatter.used_kwargs
@@ -169,6 +179,31 @@ class ResourceManager(object):
         )
         return resource
 
+    def update(self, resource_id, **fields):
+        """Updates a Resource object by resource id (can be either integer id or string identifier)"""
+        if self.resource_class.query_update is None or self.resource_class.container_update is None:
+            raise ResourceBadMethodError()
+
+        if not fields:
+            raise ResourceNoFieldsProvidedError
+
+        formatter = MemorizeFormatter()
+
+        try:
+            query_update = formatter.format(self.resource_class.query_update, resource_id, **fields)
+        except KeyError as exception:
+            param = str(exception).replace("'", "")
+
+            if param in self.params:
+                fields[param] = self.params[param]
+                query_update = formatter.format(self.resource_class.query_update, resource_id, **fields)
+            else:
+                raise ValidationError('{0} argument is required'.format(exception))
+
+        url = '{0}{1}'.format(self.redmine.url, query_update)
+        data = {self.resource_class.container_update: formatter.unused_kwargs}
+        return self.redmine.request('put', url, data=data)
+
     def delete(self, resource_id, **params):
         """Deletes a Resource object by resource id (can be either integer id or string identifier)"""
         if self.resource_class.query_delete is None:
