@@ -1,7 +1,7 @@
 from datetime import datetime
 from redmine.utilities import to_string
 from redmine.managers import ResourceManager
-from redmine.exceptions import ResourceAttrError
+from redmine.exceptions import ResourceAttrError, ReadonlyAttrError, CustomFieldValueError
 
 # Resources which when accessed from some other
 # resource should become a ResourceSet object
@@ -33,6 +33,7 @@ _RESOURCE_MAP = {
     'priority': 'Enumeration',
     'activity': 'Enumeration',
     'category': 'IssueCategory',
+    'fixed_version': 'Version',
 }
 
 # Resources which when access from some other
@@ -48,33 +49,69 @@ _RESOURCE_RELATIONS_MAP = {
     'issues': 'Issue',
 }
 
+# Resource attributes which when set should
+# also set another resource id to its value
+_RESOURCE_SINGLE_ATTR_ID_MAP = {
+    'parent_id': 'parent',
+    'project_id': 'project',
+    'tracker_id': 'tracker',
+    'priority_id': 'priority',
+    'assigned_to_id': 'assigned_to',
+    'category_id': 'category',
+    'fixed_version_id': 'fixed_version',
+    'parent_issue_id': 'parent',
+    'issue_id': 'issue',
+    'activity_id': 'activity',
+}
+
+# Resource attributes which when set should
+# also set another resource ids to their value
+_RESOURCE_MULTIPLE_ATTR_ID_MAP = {
+    'user_ids': 'users',
+    'role_ids': 'roles',
+}
+
 
 class _Resource(object):
     """Implementation of Redmine resource"""
+    manager = None
+    attributes = {}
+
     redmine_version = None
     container_all = None
     container_one = None
     container_filter = None
     container_create = None
+    container_update = None
     query_all = None
     query_one = None
     query_filter = None
     query_create = None
+    query_update = None
     query_delete = None
 
-    _relations = {}
+    _changes = {}
+    _includes = ()
+    _relations = ()
+    _readonly = ('id', 'created_on', 'updated_on', 'author', 'user', 'project', 'issue')
     __length_hint__ = None  # fixes Python 2.6 list() call on resource object
 
     def __init__(self, manager, attributes):
         """Accepts manager instance object and resource attributes dict"""
         self.manager = manager
-        self.attributes = self._relations.copy()
+        self.attributes = dict((include, None) for include in self._includes)
+        self.attributes.update(dict((relation, None) for relation in self._relations))
         self.attributes.update(attributes)
+        self._readonly += self._relations + self._includes
 
     def __getitem__(self, item):
         """Provides a dictionary like access to resource attributes"""
         return getattr(self, item)
 
+    def __setitem__(self, item, value):
+        """Provides a dictionary like setter for resource attributes"""
+        return setattr(self, item, value)
+
     def __getattr__(self, item):
         """Returns the requested attribute and makes a conversion if needed"""
         if item in self.attributes:
@@ -88,11 +125,17 @@ class _Resource(object):
                 manager = ResourceManager(self.manager.redmine, _RESOURCE_SET_MAP[item])
                 return manager.to_resource_set(self.attributes[item])
 
-            # If item should be requested from Redmine, let's do it
-            elif item in _RESOURCE_RELATIONS_MAP and self.attributes[item] is None:
-                filters = {'{0}_id'.format(self.__class__.__name__.lower()): self.id}
+            # If item is a relation and should be requested from Redmine, let's do it
+            elif item in self._relations and self.attributes[item] is None:
+                filters = {'{0}_id'.format(self.__class__.__name__.lower()): self.internal_id}
                 manager = ResourceManager(self.manager.redmine, _RESOURCE_RELATIONS_MAP[item])
-                return manager.filter(**filters)
+                self.attributes[item] = manager.filter(**filters)
+                return self.attributes[item]
+
+            # If item is an include and should be requested from Redmine, let's do it
+            elif item in self._includes and self.attributes[item] is None:
+                self.attributes[item] = self.refresh(include=item).attributes[item]
+                return getattr(self, item)
 
         try:
             # If the requested item is a date/datetime string
@@ -106,11 +149,84 @@ class _Resource(object):
         except ValueError:
             return self.attributes[item]
         except KeyError:
-            raise ResourceAttrError()
+            raise_attr_exception = self.manager.redmine.raise_attr_exception
+
+            if isinstance(raise_attr_exception, bool) and raise_attr_exception:
+                raise ResourceAttrError()
+            elif isinstance(raise_attr_exception, (list, tuple)) and self.__class__.__name__ in raise_attr_exception:
+                raise ResourceAttrError()
+
+            return None
+
+    def __setattr__(self, item, value):
+        """Sets the requested attribute"""
+        if item in self.__class__.__base__.__dict__:
+            super(_Resource, self).__setattr__(item, value)
+        elif item in self._readonly:
+            raise ReadonlyAttrError()
+        elif item == 'custom_fields':
+            for org_index, org_field in enumerate(self.attributes.setdefault('custom_fields', [])):
+                if 'value' not in org_field:
+                    self.attributes['custom_fields'][org_index]['value'] = '0'
+
+                try:
+                    for new_index, new_field in enumerate(value):
+                        if org_field['id'] == new_field['id']:
+                            self.attributes['custom_fields'][org_index]['value'] = value.pop(new_index)['value']
+                except (TypeError, KeyError):
+                    raise CustomFieldValueError()
+
+            self.attributes['custom_fields'].extend(value)
+            self._changes[item] = self.attributes['custom_fields']
+        else:
+            self._changes[item] = value
+            self.attributes[item] = value
+
+            if item in _RESOURCE_SINGLE_ATTR_ID_MAP:
+                self.attributes[_RESOURCE_SINGLE_ATTR_ID_MAP[item]] = {'id': value}
+            elif item in _RESOURCE_MULTIPLE_ATTR_ID_MAP:
+                self.attributes[_RESOURCE_MULTIPLE_ATTR_ID_MAP[item]] = [{'id': member_id} for member_id in value]
 
     def refresh(self, **params):
         """Reloads resource data from Redmine"""
-        return self.manager.get(self.id, **params)
+        return self.manager.get(self.internal_id, **params)
+
+    def pre_create(self):
+        """Tasks that should be done before creating the resource"""
+        pass
+
+    def post_create(self):
+        """Tasks that should be done after creating the resource"""
+        pass
+
+    def pre_update(self):
+        """Tasks that should be done before updating the resource"""
+        pass
+
+    def post_update(self):
+        """Tasks that should be done after updating the resource"""
+        pass
+
+    def save(self):
+        """Creates or updates a resource"""
+        if any(item in self.attributes and item not in (self._relations + self._includes) for item in self._readonly):
+            self.pre_update()
+            self.manager.update(self.internal_id, **self._changes)
+            self.attributes['updated_on'] = datetime.utcnow().strftime(self.manager.redmine.datetime_format)
+            self.post_update()
+        else:
+            self.pre_create()
+            for item, value in self.manager.create(**self._changes):
+                self.attributes[item] = value
+            self.post_create()
+
+        self._changes = {}
+        return True
+
+    @classmethod
+    def translate_params(cls, params):
+        """Translates internal param names to the real Redmine param names if needed"""
+        return params
 
     @property
     def url(self):
@@ -153,19 +269,16 @@ class Project(_Resource):
     container_all = 'projects'
     container_one = 'project'
     container_create = 'project'
+    container_update = 'project'
     query_all = '/projects.json'
     query_one = '/projects/{0}.json'
     query_create = '/projects.json'
+    query_update = '/projects/{0}.json'
     query_delete = '/projects/{0}.json'
 
-    _relations = {
-        'wiki_pages': None,
-        'memberships': None,
-        'issue_categories': None,
-        'versions': None,
-        'news': None,
-        'issues': None,
-    }
+    _includes = ('trackers', 'issue_categories')
+    _relations = ('wiki_pages', 'memberships', 'issue_categories', 'versions', 'news', 'issues')
+    _readonly = _Resource._readonly + ('identifier',)
 
 
 class Issue(_Resource):
@@ -174,16 +287,29 @@ class Issue(_Resource):
     container_one = 'issue'
     container_filter = 'issues'
     container_create = 'issue'
+    container_update = 'issue'
     query_all = '/issues.json'
     query_one = '/issues/{0}.json'
     query_filter = '/issues.json'
     query_create = '/projects/{project_id}/issues.json'
+    query_update = '/issues/{0}.json'
     query_delete = '/issues/{0}.json'
 
-    _relations = {
-        'relations': None,
-        'time_entries': None,
-    }
+    _includes = ('children', 'attachments', 'relations', 'changesets', 'journals', 'watchers')
+    _relations = ('relations', 'time_entries')
+    _readonly = _Resource._readonly + ('spent_hours',)
+
+    def __getattr__(self, item):
+        if item == 'version':
+            return super(Issue, self).__getattr__('fixed_version')
+
+        return super(Issue, self).__getattr__(item)
+
+    def __setattr__(self, item, value):
+        if item == 'version_id':
+            return super(Issue, self).__setattr__('fixed_version_id', value)
+
+        return super(Issue, self).__setattr__(item, value)
 
     def __str__(self):
         try:
@@ -213,12 +339,24 @@ class TimeEntry(_Resource):
     container_one = 'time_entry'
     container_filter = 'time_entries'
     container_create = 'time_entry'
+    container_update = 'time_entry'
     query_all = '/time_entries.json'
     query_one = '/time_entries/{0}.json'
-    query_filter = '/issues/{issue_id}/time_entries.json'
+    query_filter = '/time_entries.json'
     query_create = '/time_entries.json'
+    query_update = '/time_entries/{0}.json'
     query_delete = '/time_entries/{0}.json'
 
+    @classmethod
+    def translate_params(cls, params):
+        if 'from_date' in params:
+            params['from'] = params.pop('from_date')
+
+        if 'to_date' in params:
+            params['to'] = params.pop('to_date')
+
+        return params
+
     def __str__(self):
         return str(self.id)
 
@@ -272,13 +410,21 @@ class WikiPage(_Resource):
     container_filter = 'wiki_pages'
     container_one = 'wiki_page'
     container_create = 'wiki_page'
+    container_update = 'wiki_page'
     query_filter = '/projects/{project_id}/wiki/index.json'
     query_one = '/projects/{project_id}/wiki/{0}.json'
     query_create = '/projects/{project_id}/wiki/{title}.json'
+    query_update = '/projects/{project_id}/wiki/{0}.json'
     query_delete = '/projects/{project_id}/wiki/{0}.json'
 
-    def refresh(self):
-        return self.manager.get(self.title, project_id=self.manager.params['project_id'])
+    _includes = ('attachments',)
+    _readonly = _Resource._readonly + ('version',)
+
+    def refresh(self, **params):
+        return super(WikiPage, self).refresh(**dict(params, project_id=self.manager.params.get('project_id', 0)))
+
+    def post_update(self):
+        self.attributes['version'] = self.attributes.get('version', 0) + 1
 
     @property
     def internal_id(self):
@@ -302,12 +448,16 @@ class ProjectMembership(_Resource):
     redmine_version = '1.4'
     container_filter = 'memberships'
     container_one = 'membership'
+    container_update = 'membership'
     container_create = 'membership'
     query_filter = '/projects/{project_id}/memberships.json'
     query_one = '/memberships/{0}.json'
     query_create = '/projects/{project_id}/memberships.json'
+    query_update = '/memberships/{0}.json'
     query_delete = '/memberships/{0}.json'
 
+    _readonly = _Resource._readonly + ('user', 'roles')
+
     def __str__(self):
         return str(self.id)
 
@@ -323,10 +473,12 @@ class IssueCategory(_Resource):
     redmine_version = '1.3'
     container_filter = 'issue_categories'
     container_one = 'issue_category'
+    container_update = 'issue_category'
     container_create = 'issue_category'
     query_filter = '/projects/{project_id}/issue_categories.json'
     query_one = '/issue_categories/{0}.json'
     query_create = '/projects/{project_id}/issue_categories.json'
+    query_update = '/issue_categories/{0}.json'
     query_delete = '/issue_categories/{0}.json'
 
 
@@ -356,11 +508,22 @@ class Version(_Resource):
     container_filter = 'versions'
     container_one = 'version'
     container_create = 'version'
+    container_update = 'version'
     query_filter = '/projects/{project_id}/versions.json'
     query_one = '/versions/{0}.json'
     query_create = '/projects/{project_id}/versions.json'
+    query_update = '/versions/{0}.json'
     query_delete = '/versions/{0}.json'
 
+    def __getattr__(self, item):
+        # We have to return status attribute as it is, otherwise it
+        # will be automatically converted to IssueStatus resource
+        # by the parent _Resource object which is not what we want
+        if item == 'status':
+            return self.attributes[item]
+
+        return super(Version, self).__getattr__(item)
+
 
 class User(_Resource):
     redmine_version = '1.1'
@@ -368,12 +531,17 @@ class User(_Resource):
     container_one = 'user'
     container_filter = 'users'
     container_create = 'user'
+    container_update = 'user'
     query_all = '/users.json'
     query_one = '/users/{0}.json'
     query_filter = '/users.json'
     query_create = '/users.json'
+    query_update = '/users/{0}.json'
     query_delete = '/users/{0}.json'
 
+    _includes = ('memberships', 'groups')
+    _readonly = _Resource._readonly + ('api_key', 'last_login_on', 'custom_fields')
+
     def __str__(self):
         try:
             return super(User, self).__str__()
@@ -398,11 +566,15 @@ class Group(_Resource):
     container_all = 'groups'
     container_one = 'group'
     container_create = 'group'
+    container_update = 'group'
     query_all = '/groups.json'
     query_one = '/groups/{0}.json'
     query_create = '/groups.json'
+    query_update = '/groups/{0}.json'
     query_delete = '/groups/{0}.json'
 
+    _includes = ('memberships', 'users')
+
 
 class Role(_Resource):
     redmine_version = '1.4'
@@ -417,7 +589,7 @@ class News(_Resource):
     container_all = 'news'
     container_filter = 'news'
     query_all = '/news.json'
-    query_filter = '/projects/{project_id}/news.json'
+    query_filter = '/news.json'
 
 
 class IssueStatus(_Resource):
